package com.jay.monitor.data.server.factory.generate;

import com.jay.monitor.data.server.enums.DataSplitType;
import com.jay.monitor.data.server.factory.IdGenerateService;
import org.springframework.stereotype.Component;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.function.Consumer;

/**
 * @Module 编号生成器
 * @Description 根据天生成编号
 * @Author liukaixiong
 * @Date 2020/11/5 15:04
 */
@Component
public class DayIdGenerate implements IdGenerateService {

    public static final long DAY = 24 * 3600 * 1000L;

    private SimpleDateFormat sdf = new SimpleDateFormat("yyMMdd");

    private volatile long m_timestamp = getDayTimestamp();

    private volatile String defaultId = resetCounter(m_timestamp);

    public void setM_timestamp(long m_timestamp) {
        this.m_timestamp = m_timestamp;
    }

    public void setDefaultId(String defaultId) {
        this.defaultId = defaultId;
    }

    public long getM_timestamp() {
        return m_timestamp;
    }

    public String getDefaultId() {
        return defaultId;
    }

    protected long getDayTimestamp() {
        long timestamp = System.currentTimeMillis();
        return timestamp / DAY;
    }

    private String resetCounter(long timestamp) {
        this.m_timestamp = timestamp;
        this.defaultId = sdf.format(new Date(System.currentTimeMillis()));
        return this.defaultId;
    }

    @Override
    public DataSplitType splitType() {
        return DataSplitType.DAY;
    }

    @Override
    public String getCurrentId(Consumer<String> consumer) {
        long timestamp = getDayTimestamp();
        String id = defaultId;
        if (timestamp != m_timestamp) {
            synchronized (this) {
                if (timestamp != m_timestamp) {
                    id = resetCounter(timestamp);
                    consumer.accept(id);
                }
            }
        }
        return id;
    }

    @Override
    public String getNextId(String id) {
        return null;
    }
}
