package com.jay.monitor.data.server.factory.generate;

import com.jay.monitor.data.server.enums.DataSplitType;
import com.jay.monitor.data.server.factory.IdGenerateService;
import org.springframework.stereotype.Component;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.function.Consumer;

/**
 * @Module 编号生成器
 * @Description 根据小时生成
 * @Author liukaixiong
 * @Date 2020/11/5 19:13
 */
@Component
public class HourIdGenerate implements IdGenerateService {

    private SimpleDateFormat sdf = new SimpleDateFormat("yyMMddHH");

    public static final long HOUR = 3600 * 1000L;

    private volatile long hourTimestamp = getHourTimestamp();

    private volatile String defaultId = resetCounter(hourTimestamp);

    protected long getHourTimestamp() {
        long timestamp = System.currentTimeMillis();
        return timestamp / HOUR;
    }

    private String resetCounter(long timestamp) {
        hourTimestamp = timestamp;
        return sdf.format(new Date(System.currentTimeMillis()));
    }

    @Override
    public DataSplitType splitType() {
        return DataSplitType.HOUR;
    }

    @Override
    public String getCurrentId(Consumer<String> consumer) {
        long timestamp = getHourTimestamp();
        String id = defaultId;
        if (timestamp != hourTimestamp) {
            synchronized (this) {
                if (timestamp != hourTimestamp) {
                    id = resetCounter(timestamp);
                    consumer.accept(id);
                }
            }
        }
        return id;
    }

    @Override
    public String getNextId(String id) {
        return null;
    }
}
